"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_1 = __importDefault(require("request"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
// Connections to services can frequently slow down due to load, congestion, maintenance,
// degradation, shaping, issues, etc.
// We don't want a part upload to abort halfway without a good reason as it might be a
// large part.
const HTTP_REQUEST_TIMEOUT = 120000; // 2 minutes.
// Also set the AWS default HTTP request timeouts.
aws_sdk_1.default.config.httpOptions = {
    connectTimeout: HTTP_REQUEST_TIMEOUT,
    timeout: HTTP_REQUEST_TIMEOUT,
};
class HTTPBadStatusError extends Error {
    constructor(status, body, url) {
        super(`${url} returned bad HTTP status of ${status}: ${body}`);
        this.statusCode = status;
    }
}
exports.HTTPBadStatusError = HTTPBadStatusError;
const isPlainObject = (o) => {
    if (o == null) {
        return false;
    }
    const proto = Object.getPrototypeOf(o);
    return proto == Object.prototype || proto == null;
};
exports.http = ({ method, url, headers, body }) => {
    return new Promise((resolve, reject) => {
        if (isPlainObject(body)) {
            body = JSON.stringify(body);
        }
        request_1.default({
            url,
            method,
            headers,
            body,
            timeout: HTTP_REQUEST_TIMEOUT,
        }, (err, res) => {
            if (err) {
                reject(err);
            }
            else if (res.statusCode < 200 || res.statusCode > 299) {
                reject(new HTTPBadStatusError(res.statusCode, res.body, url));
            }
            else {
                const parsed = JSON.parse(res.body);
                resolve(parsed);
            }
        });
    });
};
//# sourceMappingURL=http.js.map